/***********************************************************************
<@File>
Name		   = kurt_sim_client.c
Description  = Client for sending Data to USARSim
Author	   = Sven Albrecht and Jochen Sprickerhof
Date		   = 2005
</@File>
***********************************************************************/
#include "sim/kurt_sim_client.h"

kurt_sim_client sim_client;
///Config file which stores the information about wheel circumference, max. speed, etc.
extern ConfigFileKurt MyConfigFileKurt;
/// string for socket communication
char send_string[60];
int send_length;
int connected=0;

// Thread
pthread_t thread_id;
pthread_attr_t sim_attr;
pthread_mutex_t parse_socket_mut = PTHREAD_MUTEX_INITIALIZER;

void *parse_socket(void*);
int c_sock;

/// Ticks Encoder
int newLeft = 0, newRight = 0, oldLeft = 0, oldRight = 0, ticks = 0;
/// real position x,y,z only for coparision with encoder data
float realX = 0.0, realY = 0.0, realZ = 0.0;
/// actual time encoder querys
long double newEncoderTime = 0.0;
/// time since last encoder query
long double oldEncoderTime = 0.0;
/// SICK range buffer with sick header
char sim_range[372] = {0x02, 0x80, 0x6c, 0x01, 0xB0, 0xb5};
int laser_pointer = 0;
long double sick_time=0;
int received_sim_scan;
/// current horizontal laserscanner rotation
int scanner_rot;
///Kurts orientation obtaind by the INU sensor
float inu_rotX = 0.0, inu_rotY = 0.0, inu_rotZ = 0.0;
/*
 * Init Sim Client for Data
 */
void kurt_sim_client::init_sim_client(int port) {
  ticks = MyConfigFileKurt.Get_Ticks_Per_Turn_Of_Wheel()/360;
  
  // Init Client
  cout<<"USAR_SIM_CLIENT: Init Sim Client\n";

  // Create Socket
  c_sock = socket(AF_INET,SOCK_STREAM,0);
  if (c_sock == -1) {
    cout << "USAR_SIM_CLIENT Error: No Socket" << endl;
    exit(0);
  }

  // Create Connection
  c_addr.sin_addr.s_addr = inet_addr(MyConfigFileKurt.Get_USAR_Server_IP());
  c_addr.sin_port = htons(port);
  c_addr.sin_family = AF_INET;

  // Connect over Port
  cout << "USAR_SIM_CLIENT: Waiting for Connection on Port: " << port << flush;
  while (connect(c_sock,(struct sockaddr*)&c_addr,sizeof(c_addr))!=0) {
    cout << "." << flush;
    usleep(1000000);
  }
  cout<<"\n";

  // Init Robot
  cout << "USAR_SIM_CLIENT: Connected to Server" <<endl;
  send_length=sprintf(send_string,
				  "INIT {ClassName %s} {Name R1} {Location %s}\r\n",
				  MyConfigFileKurt.Get_USAR_Robot(),
				  MyConfigFileKurt.Get_USAR_Location());
  cout<<send_string<<endl;
  if(send(c_sock,send_string, send_length, 0) < 1) {
    cout << "USAR_SIM_CLIENT Error: Could not spawn Robot" << endl;
  }

  //create parser Thread
  pthread_attr_init (&sim_attr);
  pthread_create(&thread_id, &sim_attr, parse_socket, 0);

  connected=1;
  received_sim_scan = 0;
}
/*
 * Close USARSim connection
 */
void kurt_sim_client::close_sim_client() {
  close(c_sock);
  cout << "USAR_SIM_CLIENT: Close Unreal Socket" << endl;
}
/*
 *
 */
int kurt_sim_client::set_wheel_speed1(float v_l,float v_r) {
  //USARSim v_l = radians per second
  // m/s -> rad/s 
  //
  float wheelCirc = MyConfigFileKurt.Get_Wheel_CircUmf()/100.0;
  // compute wheel rotation in rad
  float wheelRotLeft =rad(v_l/wheelCirc*360.0);
  float wheelRotRight =rad(v_r/wheelCirc*360.0);
  send_length = sprintf(send_string,
				    "DRIVE {Left %f} {Right %f} {Normalized True}\r\n",wheelRotLeft,wheelRotRight);
  if(send(c_sock, send_string, send_length, 0) < 0) {
    cout << "USAR_SIM_CLIENT: Connection to Server lost\n";
  }
  return 0;
}
/*
 * Set the different Servos
 */
int kurt_sim_client::setServo(int servonum, int value) {
  if(!connected)
    return 0;
  switch(servonum){
  case 1:  //Scanner
    send_length=sprintf(send_string,"DRIVE {Name ScannerSides} {Value %f}\r\n",value<0 ? 0.02152 : -0.0157);
    usleep(250000);
    break;
  case 2:  //Camera left up/down
    send_length=sprintf(send_string,"MISPKG {Name Cam2} {Rotation 0,0,%f} {Order 0}\r\n",value<0 ? 0.175:-0.175);
    break;
  case 3:  //Camera left left/right
    send_length=sprintf(send_string,"MISPKG {Name Cam2} {Rotation 0,%f,0} {Order 0}\r\n",value<0 ? 0.175:-0.175);
    break;
  case 4:  //Camera right up/down
    send_length=sprintf(send_string,"MISPKG {Name Cam1} {Rotation 0,0,%f} {Order 0}\r\n",value<0 ? 0.175:-0.175);
    break;
  case 5:  //Camera right left/right
    send_length=sprintf(send_string,"MISPKG {Name Cam1} {Rotation 0,%f,0} {Order 0}\r\n",value<0 ? 0.175:-0.175);
    break;
  default:
    return 0;
  }
  //cout << send_string <<endl;  
  if(send(c_sock, send_string, send_length, 0) < 0) {
    cout << "USAR_SIM_CLIENT: Connection to Server lost\n";
  }
  return 0;
}
/*
 *
 */
void kurt_sim_client::read_wheel_encoder(long *left, long *right,long double *timediff)
{
  static int allticksleft=0;
  static int allticksright=0;
  // Catch tick jump at 360
  // This method is called so frequently, that the wheel could not have turned more than
  // 360
  if(abs(newLeft - oldLeft) > 200 && oldLeft > 0){     //left wheel moves forward
    oldLeft = oldLeft - 360;
  }
  if(abs(newRight - oldRight) > 200 && oldRight > 0){  //right wheel moves forward
    oldRight = oldRight - 360;
  }
  if(abs(newLeft - oldLeft) > 200 && oldLeft < 0){     //left wheel moves backward
    oldLeft = oldLeft + 360;
  }
  if(abs(newRight - oldRight) > 200 && oldRight < 0){  //right wheel moves forward
    oldRight = oldRight + 360;
  }
  *left  = (newLeft - oldLeft) * ticks;
  allticksleft += (newLeft - oldLeft) * ticks;
  allticksright += (newRight - oldRight) * ticks;
  *right = (newRight - oldRight) * ticks;
  
  if(*left != 0 || *right !=0 ){
    /*
	 cout<<"wheel encoder left " << *left        << " right "    << *right   << endl <<
	 "newLeft "           << newLeft      << " newRight " << newRight << endl <<
	 "oldLeft "           << oldLeft      << " oldRight " << oldRight << endl <<
	 "allTicksLeft "      << allticksleft <<  " allTicksRight "      << allticksright << endl<<
	 "realX "             << realX        << " realY  "   << realY    << " realZ "<< realZ <<endl;
    */
    *timediff = (newEncoderTime - oldEncoderTime)*1000; //milli seconds
    //ut<<"oldencoder "<<oldEncoderTime<<" newEncoderTime "<< newEncoderTime << " TimeDiff "<< *timediff
    // << endl << endl;
    oldEncoderTime = newEncoderTime;
    oldLeft  = newLeft;
    oldRight = newRight;
  }
}
/*
 * 
 */
int kurt_sim_client::SICK_read(int fd, char buf[], size_t count) {
  if (received_sim_scan == 1) {
    pthread_mutex_lock(&parse_socket_mut);
    for(unsigned int i = 0; i < count; i++) {
      buf[i] = sim_range[laser_pointer++];
      laser_pointer %= 372;
    }
    pthread_mutex_unlock(&parse_socket_mut);
    received_sim_scan = 0;
    return count;
  }
  else
    return 0;
}
/**
 * This pointer function parses the USARSIM status and sensor string
 * and initializes the correspondingly variables with the given data.
 * This function has its own thread. 
 */
void *parse_socket(void* arg) {
  int i;
  char rec[1], buff[10240];
  int abstand;
  char* pch;

  while(1) {
    for(i = 0;
	   i < 10240 && recv(c_sock, rec, 1, 0) > 0 && rec[0] != '\n';
	   i++) {
      buff[i]=rec[0];
    }
    buff[i-1] = '\0';
    if(buff[1] == 'E') { // SEN Message
      if(strstr(buff, "Scanner1") != NULL) { // Sick Data
	   pch = strstr(buff, "Rotation ");
	   pch = strtok(pch, " ");
	   pch = strtok(NULL,",");
	   scanner_rot= (int)(atof(pch)); 
	   pch = strstr(buff, "Range ");
	   pch = strtok(pch, " ");
	   pthread_mutex_lock(&parse_socket_mut);
	   //usarsim 2.0.2 gibt sick andersherum zurueck deshalb
	   //hier von hinten nach vorne reinschreiben        
	   for(i = 367;
		i > 6 && (pch = strtok(NULL,",")) != NULL;
		i -= 2) {
		   abstand = (int)(1000.0 * atof(pch)); 	
		   sim_range[i] = abstand % 256;
	        sim_range[i+1] = (abstand - abstand % 256) / 256;
		}
	   pthread_mutex_unlock(&parse_socket_mut);
	   received_sim_scan = 1;
      }
      else if(strstr(buff, "Encoder")!=NULL) { //Encoder Data
	   pch=strstr(buff,"ECLeft");
	   newLeft=atoi(pch+11);
	   pch=strstr(buff,"ECRight");
	   newRight=atoi(pch+12);
	   
	   if(newLeft-oldLeft != 0 || newRight-oldRight != 0 ){ //Kurt hat sich bewegt
		pch = strstr(buff, "Time ");
		pch = strtok(pch, " ");
		pch = strtok(NULL,"}");
		newEncoderTime = (atof(pch));
	   }
	   if(newEncoderTime == 0 ){  //Zeit die Kurt rumsteht bevor irgentetwas passiert!
		pch = strstr(buff, "Time ");
		pch = strtok(pch, " ");
		pch = strtok(NULL,"}");
		oldEncoderTime = (atof(pch));
	   }
      }
	 else if(strstr(buff, "INU")!=NULL) { //INU Data
	   pch = strstr(buff, "Orientation ");
	   pch = strtok(pch, " ");
	   pch = strtok(NULL,",");
	   inu_rotX = atof(pch);
	   pch = strtok(NULL,",");
	   inu_rotY = atof(pch);
	   pch = strtok(NULL,",");
	   inu_rotZ = atof(pch);
	   //cout<<"rotX "<<inu_rotX<<" rotY "<<inu_rotY<<" rotZ "<<inu_rotZ<<endl;
	 }
    }  
    else if(buff[0]=='G') { // GEO MESSAGE
	 cout<<"current Geo information"<<endl;
	  cout<<buff<<endl;
    }
    else if(buff[1]=='T') { // STA MESSAGE
	 pch = strstr(buff, "Location ");
	 pch = strtok(pch, " ");
	 pch = strtok(NULL,",");
	 realX = (atof(pch));
	 pch = strtok(NULL,",");
	 realY = (atof(pch));
	 pch = strtok(NULL,",");
	 realZ = (atof(pch));
    }
  }
}
/*
 * This method resets the scanner to the horizontal position
 */
int kurt_sim_client::reset_scanner() {
  //get current orientation
  float diff_uu=scanner_rot-153;
  float diff_rad;
  if(diff_uu > 0) { //turn scanner up
    diff_uu-=65535;
    diff_rad = diff_uu * 360.0 / 65535.0 * 0.01745329;
    send_length=sprintf(send_string,"SET {Type Joint} {Name ScannerSides} {Opcode 0} {Params %f}\r\n",diff_rad);    
  }
  else{          //turn scanner down
    diff_rad = diff_uu * 360.0 / 65535.0 * 0.01745329;
    send_length=sprintf(send_string,"SET {Type Joint} {Name ScannerSides} {Opcode 0} {Params %f}\r\n",-diff_rad);    
  }
  
  if(send(c_sock, send_string, send_length, 0) < 0) {
    cout << "USAR_SIM_CLIENT: Connection to Server lost\n";
  }
  
  return 0;
}
/*
 * This method is called by the gyro to get the actual
 * robot rotation
 */
void kurt_sim_client::get_Theta(double *theta)
{
  *theta = -inu_rotZ;
}
/* End of File */
