#ifndef KURT_SIM_CLIENT_H
#define KURT_SIM_CLIENT_H

#include <iostream>
#include <string>
#include <fstream>

#include <fcntl.h>

#include <stdlib.h>
#include <stdio.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <asm/types.h>
#include <unistd.h>
#include <stdint.h>

#include <linux/input.h>

#include <netdb.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <fstream>

#include <pthread.h>

#include "ConfigFileKurt.h"
#include "mytime.h"
#include "scanner.icc"

using namespace std;


// Class sim_client
  /**
   * This class realizes the connection between USARSim and the kurt software.
   * The methods to get and set kurt's speed, laserscanner and cameraservos are redirected
   * to this class, if we use kurt in the simulation. 
   */
class kurt_sim_client {
public:
  /**
   * Init Client for data. Creates a socket and connects to the unreal server
   * @param port the unreal port number
   */
  void init_sim_client(int port);
  /**
   * disconnect this client
   */
  void close_sim_client();
  /**
   * Set kurts wheel speed in the simulation.
   * @param v_l left wheel speed in m/s
   * @param v_r right wheel speed in m/s
   */
  int  set_wheel_speed1(float v_l, float v_r);
  /**
   * Set the different servos (cameras, laserscanner)
   * @param servoNum the servo number (1 laserscanner, 2 left camera up/down,
   3 left camera left/right, 4 right camera up/down, 5 right camera left/right
   * @param value The value the respective servo is turned.
   */
  int setServo(int servonum, int value);
  /**
   * returns the ticks, the left respectively right wheel had turned and the
   * corresponding timediff since the last call of this function.
   */
  void read_wheel_encoder(long *left, long *right,long double *timediff);
  /**
   * This method returns the data obtained by a USARSim RangeScanner
   * using SICK range encoding.
   * @param fd
   * @param buf sick range buffer
   * @param count number of range values
   */
  int  SICK_read(int fd, char buf[], size_t count);
  /**
   * This method sets the laserscanner to the horizontal position. This method is
   * necessary because using the setServo method only approxemately resets the scanner
   * position.
   */
  int reset_scanner();
  /**
   * This method simulates Kurt's gyro using the USARSim INU sensor.
   * @param theta the current orientation angle (z axis)
   */
  void get_Theta(double *theta);
private:
  /// address for unreal server
  struct sockaddr_in c_addr;
};

#endif
/* End of File */

